@@ -1,6 +1,6 @@
 ;;;-*- Mode: Lisp; Syntax:Common-Lisp; Package:CHAOS; Base:10 -*-
 ;;;
-;;; Copyright (c) 2000-2015, Toshimi Sawada. All rights reserved.
+;;; Copyright (c) 2000-2016, Toshimi Sawada. All rights reserved.
 ;;;
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -119,9 +119,9 @@
               (|identity-rules:| :chaos-item)
               ;; ((:pred general-read-numberp))
               ((:+ prec precedence |prec:| |pecedence:|) :int)
-              (|(| (:seq-of :int) |)|)
+              (\(  (:seq-of :int)  \))
               ((:+ strat strategy |strat:| |strategy:|)
-               |(| (:seq-of :int) |)|)
+               \(  (:seq-of :int)  \))
               ((:+ l-assoc r-assoc left-associative right-associative))
               ((:+ constr ctor constructor))
               ((:+ coherent beh-coherent))
@@ -143,9 +143,9 @@
               (|identity-rules:| :chaos-item)
               ;; ((:pred general-read-numberp))
               ((:+ prec precedence |prec:| |pecedence:|) :int)
-              (|(| (:seq-of :int) |)|)
+              (\(  (:seq-of :int)  \))
               ((:+ strat strategy |strat:| |strategy:|)
-               |(| (:seq-of :int) |)|)
+               \(  (:seq-of :int)  \))
               ((:+ l-assoc r-assoc left-associative right-associative))
               ((:+ constr constructor))
               (memo)
@@ -173,9 +173,9 @@
               (|identity-rules:| :chaos-item)
               ;; ((:pred general-read-numberp))
               ((:+ prec precedence |prec:| |pecedence:|) :int)
-              (|(| (:seq-of :int) |)|)
+              (\(  (:seq-of :int)  \))
               ((:+ strat strategy |strat:| |strategy:|)
-               |(| (:seq-of :int) |)|)
+               \(  (:seq-of :int)  \))
               ((:+ l-assoc r-assoc left-associative right-associative))
               ((:+ constr constructor))
               ((:+ coherent beh-coherent))
@@ -203,9 +203,9 @@
               (|identity-rules:| :chaos-item)
               ;; ((:pred general-read-numberp))
               ((:+ prec precedence |prec:| |pecedence:|) :int)
-              (|(| (:seq-of :int) |)|)
+              (\(  (:seq-of :int)  \))
               ((:+ strat strategy |strat:| |strategy:|)
-               |(| (:seq-of :int) |)|)
+               \(  (:seq-of :int)  \))
               ((:+ l-assoc r-assoc left-associative right-associative))
               ((:+ constr constructor))
               ((:+ coherent beh-coherent))
@@ -229,8 +229,8 @@
          (|identity-rules:| :chaos-item)
          ;; ((:pred general-read-numberp))
          ((:+ prec precedence |prec:| |pecedence:|) :int)
-         (|(| (:seq-of :int) |)|)
-         ((:+ strat strategy |strat:| |strategy:|) |(| (:seq-of :int) |)|)
+         (\(  (:seq-of :int)  \))
+         ((:+ strat strategy |strat:| |strategy:|) \(  (:seq-of :int)  \))
          ((:+ l-assoc r-assoc))
          ;; ((:+ constr constructor))
          (memo)
@@ -297,13 +297,13 @@
 ;;;-----------------------------------------------------------------------------
 
   (defparameter ExDeclaration
-      '((:+ ex extending) (:if-present as :symbol) |(| :modexp |)|))
+      '((:+ ex extending) (:if-present as :symbol) \(  :modexp  \)))
   (defparameter PrDeclaration
-      '((:+ pr protecting) (:if-present as :symbol) |(| :modexp |)|))
+      '((:+ pr protecting) (:if-present as :symbol) \(  :modexp  \)))
   (defparameter UsDeclaration
-      '((:+ us using) (:if-present as :symbol) |(| :modexp |)|))
+      '((:+ us using) (:if-present as :symbol) \(  :modexp  \)))
   (defparameter IncDeclaration
-      '((:+ inc including) (:if-present as :symbol) |(| :modexp |)|))
+      '((:+ inc including) (:if-present as :symbol) \(  :modexp  \)))
 
   )
 
@@ -328,6 +328,61 @@
                        #.FoplAXDeclaration)
         |}|))
 
+  (defparameter INIT
+      '((:+ |:init| init |:init!| init!) 
+        (:if-present as :symbol)
+        (:one-of (\( (:one-of #.EqDeclaration
+                              #.CeqDeclaration
+                              #.RlDeclaration
+                              #.CRlDeclaration
+                              #.BeqDeclaration
+                              #.BCeqDeclaration
+                              #.BRLDeclaration
+                              #.BCRLDeclaration
+                              #.FoplAXDeclaration)
+                  \))
+                 (\[ (:symbol) \]))
+        |by| |{| ((:! SubstList)) |}|))
+  (defparameter USE
+      '(|:use|
+        (:one-of (\( (:seq-of :symbol) \))
+                 (\[ :symbol \] 
+                  |{| (:many-of ((:+ assoc comm associative commutative))
+                                (|id:| :chaos-item)
+                                (|identity:| :chaos-item))
+                  |}|))))
+  (defparameter EMBED
+      '(|:embed| 
+        (:one-of (\( (:seq-of :symbol) \) (:+ as into) :symbol)
+                 (\[ :symbol \]
+                  |{| (:many-of ((:+ assoc comm associative commutative))
+                                (|id:| :chaos-item)
+                                (|identity:| :chaos-item))
+                  |}| (:+ as into) :symbol))))
+
+  (defparameter IND
+      '((:+ |:ind| |:induction|)
+        (:one-of (on \( (:seq-of :term) \))
+                 (\{ (on \( (:seq-of :term) \))
+                     (base \( (:upto (|.| \)) :term)
+                           :append (:seq-of |.|
+                                      (:upto (|.| \)) :term))
+                           \))
+                  (:if-present hypo
+                   \( (:upto (|.| \)) :term )
+                           :append (:seq-of |.|
+                                            (:upto (|.| \)) :term))
+                        \))
+                  (step \( (:upto (|.| \)) :term )
+                           :append (:seq-of |.|
+                                            (:upto (|.| \)) :term))
+                        \))
+                  \}))))
+  (defparameter CITPATTR
+      '(|:theory| (:seq-of :opname) |:| :sorts -> :sort
+        |{| (:many-of ((:+ assoc comm associative commutative))
+             (|id:| :chaos-item))
+        |}|))
 )
 
 ;;;-----------------------------------------------------------------------------
@@ -458,7 +513,7 @@
 
         ;; MAKE
         ;; --------------------------------------------------
-        (make :symbol |(| :modexp |)|)
+        (make :symbol \( :modexp \))
 
         ;; TOP LEVEL COMMANDS
         ;; --------------------------------------------------
@@ -503,15 +558,11 @@
                      (:if-present  in :modexp |:|) (:seq-of :term) |.|))
         ((:+ lisp ev eval evq lispq)
          (:call (read)))
-        (;; (:+ show sh set select describe desc) ; do 
-         set
+        (set
          (:seq-of :top-opname))
-        ;; (select :modexp :args)
-        ((:+ show sh select describe desc) :args)
-        ;; (trans-chaos (:seq-of :top-opname))
+        ((:+ show sh select describe desc :sh :show) :args)
 
         ;; module elements which can appear at top(iff a module is opened.)
-
         #.PrDeclaration
         #.ExDeclaration
         #.UsDeclaration
@@ -546,7 +597,7 @@
         (close)
         (start :term |.|)
         ;; scase (<Term>) on (<Modexp>) as <Name> { <ModuleElements> } : <GoalTerm> .
-        (scase |(| (:seq-of :term) |)| in |(| :modexp |)| as :symbol |{|
+        (scase \( (:seq-of :term) \) in \( :modexp \) as :symbol |{|
                (:many-of
                 ;; MODULE IMPORTATIONS
                 ;; *NOTE*  imports { ... } is not in MANUAL, and does not have
@@ -658,7 +709,7 @@
                          (:+ within at)
                          (:one-of
                           ((:+ top it term subterm))
-                          ((:+ |(| |{| |[|)
+                          ((:+ \( |{| \[)
                            :unread
                            ((:! Selector))
                            (:seq-of of ((:! Selector)))
@@ -667,7 +718,7 @@
         ;;
         (choose (:one-of
                  ((:+ top term subterm))
-                 ((:+ |(| |{| |[|)
+                 ((:+ \( |{| \[)
                   :unread
                   ((:! Selector))
                   (:seq-of of ((:! Selector)))
@@ -707,17 +758,17 @@
         #+:bigpink (option (:one-of (reset)
                                     (= :symbol)))
         #+:bigpink ((:+ save-option save-options) :symbol)
-        #+:bigpink (flag |(| :symbol |,| (:+ on off set clear) |)|)
-        #+:bigpink (param |(| :symbol |,| :int |)|)
+        #+:bigpink (flag \(  :symbol |,| (:+ on off set clear)  \))
+        #+:bigpink (param \(  :symbol |,| :int  \))
         #+:bigpink (resolve :args)
         #+:bigpink (demod (:if-present  in :modexp |:|) (:seq-of :term) |.|)
-        #+:bigpink (sigmatch |(| :modexp |)| (:+ to with) |(| :modexp |)|)
+        #+:bigpink (sigmatch \( :modexp \) (:+ to with) \( :modexp \))
 
-        #+:bigpink (lex |(|
-                        (:upto (|,| |)|) :opname)
+        #+:bigpink (lex \(
+                        (:upto (|,| \)) :opname)
                         :append (:seq-of |,|
-                                         (:upto (|,| |)|) :opname))
-                        |)|)
+                                         (:upto (|,| \)) :opname))
+                        \))
         ;; misc toplevel commands
         (eof)
         #-CMU (#\^D)
@@ -764,26 +815,15 @@
                                 #.BCRLDeclaration
                                 #.FoplAXDeclaration)
                  |}|)
-        (|:apply| (:if-present to (:symbol)) (|(| (:seq-of :symbol) |)|))
+        (|:apply| (:if-present to (:symbol)) (\( (:seq-of :symbol) \)))
         (|:auto|)
-        (|:ind| (:+ on |:on|) |(| (:seq-of :term) |)|)
+        #.IND
         (|:roll| (:+ back |:back|))
-        (|:init| (:one-of (|(| (:one-of #.EqDeclaration
-                                        #.CeqDeclaration
-                                        #.RlDeclaration
-                                        #.CRlDeclaration
-                                        #.BeqDeclaration
-                                        #.BCeqDeclaration
-                                        #.BRLDeclaration
-                                        #.BCRLDeclaration
-                                        #.FoplAXDeclaration)
-                               |)|)
-                          (\[ (:symbol) \]))
-                 |by| |{| ((:! SubstList)) |}|)
+        #.INIT
         ((:+ |:imply| |:imp|) (\[ (:symbol) \])
                               (:one-of (|by| |{| ((:! SubstList)) |}|)
                                        (|.|)))
-        (|:cp| (:one-of (|(| (:one-of #.EqDeclaration
+        (|:cp| (:one-of (\( (:one-of #.EqDeclaration
                                       #.CeqDeclaration
                                       #.RlDeclaration
                                       #.CRlDeclaration
@@ -792,10 +832,10 @@
                                       #.BRLDeclaration
                                       #.BCRLDeclaration
                                       #.FoplGoalDeclaration)
-                             |)|)
+                             \))
                         (\[ (:symbol) \]))
                ><
-               (:one-of (|(| (:one-of #.EqDeclaration
+               (:one-of (\( (:one-of #.EqDeclaration
                                       #.CeqDeclaration
                                       #.RlDeclaration
                                       #.CRlDeclaration
@@ -804,7 +844,7 @@
                                       #.BRLDeclaration
                                       #.BCRLDeclaration
                                       #.FoplGoalDeclaration)
-                             |)|)
+                             \))
                         (\[ (:symbol) \])))
         ((:+ |:equation| |:rule|))
         (|:backward| (:+ equation rule |:equation| |:rule|))
@@ -812,13 +852,14 @@
         ((:+ |:red| |lred| |:lred| |:exec| |:bred|)
          (:rdr #..term-delimiting-chars. (:if-present  in :symbol |:|)) (:seq-of :term) |.|)
         (|:verbose| :symbol)
-        ;; (|:normalize| :symbol)
         #.CTF
         #.CSP
+        #.CITPATTR
         ((:+ |:show| |:sh| |:describe| |:desc|) :args)
         ((:+ |:def| |:define|) :symbol = (:one-of #.CTF
                                                   #.CSP
-                                                  (|(| (:seq-of :symbol) |)|)))
+                                                  #.INIT  
+                                                  (\( (:seq-of :symbol) \))))
         (|:spoiler| (:one-of (on) (off) (|.|)))
         (|:binspect|
          (:rdr #..term-delimiting-chars. (:if-present in :symbol |:|)) (:seq-of :term) |.|)
@@ -827,7 +868,15 @@
         ((:+ |:bresolve| bresolve) :args)
         ((:+ |:bshow| bshow) :args)
         ((:+ |:bg| |:bguess| bg bguess) :args)
-        (|:set| |(| :symbol |,| (:+ on off set clear ? show) |)|)
+        (|:set| \(  :symbol |,| (:+ on off set clear ? show)  \))
+        (|:order| \(
+                        (:upto (|,| \)) :opname)
+                        :append (:seq-of |,|
+                                         (:upto (|,| \)) :opname))
+                        \))
+        #.USE
+        #.EMBED
+        (|:reset|)
         ))                              ; end Top-Form
 
       ;; some separated definitions of non-terminals.
@@ -836,7 +885,7 @@
       
       (Selector (:one-of 
                  (|{| :int :append (:seq-of |,| :int) |}|)
-                 (|(| (:seq-of :int) |)|)
+                 (\(  (:seq-of :int)  \))
                  (\[ :int (:optional |..| :int) \])))
 
       ;; parameter part
@@ -849,7 +898,7 @@
       ;; Substitution
       ;;  variable-1 <- term-1; ... variable-n <- term-n;
       (SubstList ((:! Subst) :append (:seq-of (:! Subst))))
-      (Subst ((:symbol <- :term) |;|))
+      (Subst ((:symbol <- :term) \;))
       ))                                ; end of *cafeobj-scheme*
   )                                     ; end eval-when
 
