@@ -1,6 +1,6 @@
 ;;;-*-Mode:LISP; Package: CHAOS; Base:10; Syntax:Common-lisp -*-
 ;;;
-;;; Copyright (c) 2000-2015, Toshimi Sawada. All rights reserved.
+;;; Copyright (c) 2000-2018, Toshimi Sawada. All rights reserved.
 ;;;
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -72,7 +72,7 @@
 
        ;; then there may be some extensions.
        (when (and (not is-applied) (term-is-applform? term))
-         (let ((top (term-method term)))
+         (let ((top (term-head term)))
            (declare (type method top))
            (unless (let ((val (axiom-kind rule)))
                      (and val
