@@ -1,6 +1,6 @@
 ;;;-*- Mode: Lisp; Syntax: CommonLisp Package: CHAOS -*-
 ;;;
-;;; Copyright (c) 2000-2015, Toshimi Sawada. All rights reserved.
+;;; Copyright (c) 2000-2018, Toshimi Sawada. All rights reserved.
 ;;;
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -351,9 +351,9 @@
         (let ((t1 (equation-t1 equation))
               (t2 (equation-t2 equation)))
           (unless (and (term-is-application-form? t2)
-                       (method-is-of-same-operator+ (term-method t1)
-                                                    (term-method t2))
-                       (setq method (term-method t1)))
+                       (method-is-of-same-operator+ (term-head t1)
+                                                    (term-head t2))
+                       (setq method (term-head t1)))
             (return-from no-match (values nil t)))
           ;;
           (let* ((sub1 (list-assoc-subterms t1 method))
@@ -378,7 +378,7 @@
                         (push val sub1add)
                         (if (term-is-variable? image)
                             (push image sub1add)
-                          (if (eq method (setf head (term-method image)))
+                          (if (eq method (setf head (term-head image)))
                               (setq sub1add
                                 (nconc
                                  (reverse
@@ -463,10 +463,10 @@
   (declare (type term t1 t2)
            (values (or null t)))
   (if (term-is-application-form? t2)
-      (let ((hd2 (term-method t2)))
+      (let ((hd2 (term-head t2)))
         (if (method-is-of-same-operator (term-head t1)
                                         hd2)
-            (let ((l1 (list-assoc-subterms t1 (term-method t1)))
+            (let ((l1 (list-assoc-subterms t1 (term-head t1)))
                   (l2 (list-assoc-subterms t2 hd2)))
               (declare (type list l1 l2))
               (and (= (the fixnum (length l1)) (the fixnum (length l2)))
