@@ -1,6 +1,6 @@
 ;;;-*- Mode: Lisp; Syntax:CommonLisp; Package:CHAOS; Base:10 -*-
 ;;;
-;;; Copyright (c) 2000-2015, Toshimi Sawada. All rights reserved.
+;;; Copyright (c) 2000-2018, Toshimi Sawada. All rights reserved.
 ;;;
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -96,16 +96,30 @@
 
 (defvar *print-term-color* nil)
 
-(defun variable-print-string (term &optional (print-var-sort nil)
+;;; PVARIABLE-TERM? : term -> Bool
+;;; returns t if pure varible or pconstant.
+;;;
+(defun pvariable-term? (term)
+  (or (term-is-variable? term)(term-is-pconstant? term)))
+
+(defun pvariable-term-body? (term-body)
+  (or (term$is-variable? term-body) (term$is-pconstant? term-body)))
+
+;;; VARIABLE-PRINT-STRING: variable -> String
+;;;
+(defun variable-print-string (term &optional (print-var-sort t)
                                              (vars-so-far (cons nil nil)))
   (let ((name (if (numberp (variable-print-name term))
                   (format nil "_v~d" (variable-print-name term))
                 (string (variable-print-name term)))))
     (when print-var-sort
-      (unless (member term (cdr vars-so-far)
-                      :test #'variable-eq)
+      (unless (or (member term (cdr vars-so-far)
+                          :test #'variable-eq)
+                  (and (not (eq print-var-sort :always))
+                       (rassoc term (module-variables *current-module*)
+                               :test #'equal)))
         (push term (cdr vars-so-far))
-        (when (and (let ((sort (variable-sort term)))
+        (when (and '(let ((sort (variable-sort term)))
                      (not (or (eq sort *universal-sort*)
                               (eq sort *huniversal-sort*)
                               (eq sort *bottom-sort*)
@@ -113,21 +127,16 @@
                    (or *print-with-sort*
                        (not (rassoc term
                                     (module-variables *current-module*)))))
-          (let ((sn (make-array '(0) :element-type 'base-char
-                                :fill-pointer 0
-                                :adjustable t)))
-            (with-output-to-string (str sn)
-              (let ((*standard-output* str))
-                (print-sort-name (variable-sort term) *current-module*)))
+          (let ((sn (with-output-to-string (str)
+                      (let ((*standard-output* str))
+                        (print-sort-name (variable-sort term) *current-module*))
+                      str)))
             (setq name (concatenate 'string name ":" sn))))))
     name))
 
 (defun bconst-print-string (term)
-  (let ((val (term-builtin-value term))
-        (bstr (make-array '(0) :element-type 'base-char
-                          :fill-pointer 0
-                          :adjustable t)))
-    (with-output-to-string (ss bstr)
+  (let ((val (term-builtin-value term)))
+    (with-output-to-string (ss)
       (let ((*standard-output* ss))
         (if (and (bsort-p (term-sort term))
                  (bsort-term-printer (term-sort term)))
@@ -142,8 +151,8 @@
                  (princ (string val)))
                 ((characterp val)
                  (print-character val))
-                (t (princ val))))))
-    bstr))
+                (t (princ val))))
+        ss))))
 
 (defun term-to-sexpr (term &optional (print-var-sort t)
                                      (vars-so-far (cons nil nil))
@@ -157,7 +166,7 @@
                `(":var" ,(variable-print-string term print-var-sort vars-so-far)))
            (return-from term-to-sexpr
              (variable-print-string term print-var-sort vars-so-far))))
-        ((term-is-psuedo-constant? term)
+        ((term-is-pconstant? term)
          (if as-tree
              (return-from term-to-sexpr
                `(":literal" ,(variable-print-string term print-var-sort vars-so-far)))
@@ -220,12 +229,13 @@
        (= (length tok) 1)
        (assoc (char tok 0) .default-single-chars.)))
 
-(defun term-print1 (term &optional (stream *standard-output*) (print-var-sort t)
+(defun term-print1 (term &optional (stream *standard-output*) 
+                                   (print-var-sort t)
                                    (vars-so-far (cons nil nil)))
   (declare (type stream stream)
            (type (or null t) print-var-sort)
            (values t))
-  (unless (termp term)
+  (unless (term? term)
     (format stream " ~s " term)
     (return-from term-print1))
   (let ((*standard-output* stream)
@@ -242,7 +252,7 @@
     (when (and (term-is-red term) *print-term-color*)
       (princ "r::"))
     ;; 
-    (cond ((or (term$is-variable? body) (term$is-psuedo-constant? body))
+    (cond ((pvariable-term? term)
            (let ((vstr (variable-print-string term print-var-sort vars-so-far)))
              (princ vstr stream)))
           ((term$is-system-object? body)
@@ -264,7 +274,7 @@
            (let ((*print-pretty* t))
              (format t "~s" (term$lisp-form-original-form body))))
           ((term$is-applform? body)
-           (let* ((hd (term$head body))
+           (let* ((hd (appl$head body))
                   (op (method-operator hd)))
              (cond ((not (operator-is-mixfix op))
                     (princ (format nil "~{~a~^ ~}" (operator-symbol op)) stream)
@@ -309,7 +319,7 @@
            (type fixnum prec)
            (type stream stream)
            (type (or null t) print-var-sort))
-  (unless (termp term)
+  (unless (term? term)
     (format stream " ~s " term)
     (return-from term-print2))
   (let ((*standard-output* stream)
@@ -324,10 +334,10 @@
     (when (print-check .file-col. 0 stream)             ; 20?? 
       (setq .file-col. (file-column stream)))
     ;;
-    (when (and (term-is-red term) *print-term-color*)
+    (when (and *print-term-color* (term-is-red term))
       (princ "r::" stream))
 
-    (cond ((or (term-is-variable? term) (term-is-psuedo-constant? term))
+    (cond ((pvariable-term? term)
            (let ((vstr (variable-print-string term
                                               print-var-sort
                                               vars-so-far)))
@@ -383,7 +393,7 @@
                         (princ ")" stream))))
                    ;; mix fix operators
                    (t (let ((prec-test (and (get-method-precedence hd)
-                                            (<= prec (get-method-precedence hd))))
+                                            (< prec (get-method-precedence hd)))) ; was <=
                             (assoc-test (method-is-associative hd))
                             (token-seq (operator-token-sequence
                                         (method-operator hd)))
@@ -431,6 +441,9 @@
            (type (or t stream))
            (type (or null t) print-var-sort)
            (values t))
+  (unless *current-module*
+    (with-output-chaos-error ('no-context)
+      (format t "Internal error: printing term, no current context")))
   (when (eq stream t)
     (setq stream *standard-output*))
   (let* ((vars-so-far (cons nil .printed-vars-so-far.))
@@ -464,18 +477,16 @@
            (type stream stream)
            (values t))
   (let ((*print-indent* (+ *print-indent* 2))
-        (**print-var-sort** nil)
+        (**print-var-sort** t)
         (paren? nil))
     (setq paren?
           (case *print-xmode*
             (:fancy
-             (if (or (term-is-variable? term)
-                     (term-is-psuedo-constant? term))
+             (if (pvariable-term? term)
                  nil
                t))
             (:normal
-             (if (or (term-is-variable? term)
-                     (term-is-psuedo-constant? term))
+             (if (pvariable-term? term)
                  nil
                (if (and (term-is-applform? term)
                         (operator-is-mixfix (method-operator
@@ -483,6 +494,10 @@
                    nil
                  t)))
             (otherwise  nil)))
+    (when (and (or (eq *print-xmode* :fancy)
+                   (eq *print-xmode* :normal))
+               (pvariable-term? term))
+      (setq **print-var-sort** nil))
     ;;
     (when paren? (princ "(" stream))
     (term-print term stream)
@@ -500,7 +515,7 @@
                          :fill-pointer 0
                          :adjustable t)))
     (with-output-to-string (s str)
-      (term-print-with-sort term str))
+      (term-print-with-sort term s))
     str))
 
 ;;;-----------------------------------------------------------------------------
@@ -534,8 +549,7 @@
 (defun !print-term-tree (tree show-sort stream &optional (show-as-graph t))
   (let* ((*show-sort* show-sort)
          (leaf?
-          #'(lambda (tree) (or (term-is-variable? tree)
-                               (term-is-constant? tree))))
+          #'(lambda (tree) (pvariable-term? tree)))
          (leaf-name
           #'(lambda (term)
               (let ((name (if (term-is-builtin-constant? term)
@@ -545,7 +559,7 @@
                                       ((symbolp value)
                                        (string value))
                                       (t (format nil "(~s)" value))))
-                            (if (term-is-variable? term)
+                            (if (pvariable-term? term)
                                 (string (variable-print-name term))
                               (if (term-is-lisp-form? term)
                                   (lisp-form-original-form term)
