@@ -1,6 +1,6 @@
 ;;;-*- Mode:Lisp; Syntax:Common-Lisp; Package:CHAOS -*-
 ;;;
-;;; Copyright (c) 2000-2015, Toshimi Sawada. All rights reserved.
+;;; Copyright (c) 2000-2018, Toshimi Sawada. All rights reserved.
 ;;;
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -92,22 +92,25 @@
          ,@body))))
 
 ;;;
+;;; IS-DIRECTORY? path -> (or null pathname)
 ;;;
-;;;
-
 (defun is-directory? (path)
   (declare (type (or pathname simple-string) path))
   (let ((dpath (expand-file-name path)))
     #+(or GCL CMU :openmcl)
     (probe-file (concatenate 'string dpath "/"))
     #+:SBCL
-    (let ((directory-delimiter "/")  ; sbcl uses / on all platforms!
-          (p (probe-file dpath)))
-      (if p
-          (and (string-equal (subseq (namestring p)
-                                     (1- (length (namestring p))))
-                             directory-delimiter)
-               p)
+    (let* ((p (probe-file dpath))
+           (nstr (if p 
+                     (namestring p)
+                   "")))
+      (declare (type simple-string nstr))
+      (if (and p
+               (string-equal (subseq nstr
+                                     (1- (length nstr)))
+                             ;; sbcl uses "/" on all platforms.
+                             "/"))
+          p
         nil))
     #+:Allegro
     (if (excl:file-directory-p dpath)
@@ -120,103 +123,129 @@
           p
         nil))))
 
-(defun is-simple-file-name? (path)
-  (declare (type (or simple-string pathname) path)
-           (values (or null t)))
-  (if (pathnamep path)
-      (null (pathname-directory path))
-    (if (stringp path)
-        (let ((dir (pathname-directory (pathname path))))
-          (if (or (null dir)
-                  (equal '(:relative) dir))
-              t
-            nil))
-      (error "is-simple-file-name? : given non string arg ~a" path))))
-
 (defun is-relative-file-name? (path)
-  (declare (type (or simple-string pathname) path)
-           (values (or null t)))
+  (declare (type (or simple-string pathname) path))
   (let ((pn (if (pathnamep path)
 		path
-	      (if (stringp path)
-		  (pathname path)
-		(error "is-relative-file-name? : given non string arg ~a" path)))))
-    (if (or (null (pathname-directory pn))
-	   (member ':relative (pathname-directory pn) :test #'equal))
-	t
-      nil)))
+              (pathname path))))
+    (declare (type pathname pn))
+    (or (null (pathname-directory pn))
+        (member ':relative (pathname-directory pn) :test #'equal))))
 
 (defun supply-suffixes (path suffixes)
   (declare (type (or simple-string pathname) path)
-           (type list suffixes)
-           (values list))
+           (type list suffixes))
   #+:SBCL
-  (when (position #\. (if (pathnamep path)
-                          (pathname-name path)
-                        path) 
-                  :from-end t)
-    (return-from supply-suffixes (list path)))
-  (mapcar #'pathname
-          (mapcar #'(lambda (x) (concatenate 'string
-                                  (namestring path)
-                                  (namestring x)))
-                  suffixes)))
+   (let ((pn (if (pathnamep path)
+                 (pathname-name path)
+               path)))
+     (declare (type simple-string pn))
+     (when (position #\. pn :from-end t)
+       (return-from supply-suffixes (list path))))
+   (mapcar #'pathname
+            (mapcar #'(lambda (x) (concatenate 'string
+                                    (namestring path)
+                                    (namestring x)))
+                     suffixes)))
+
+(defun chaos-get-relative-path (f-name)
+  (declare (type simple-string f-name)
+           (values pathname))
+  (flet ((chaos-relative-pathname? ()
+           (let ((fdp (pathname-directory (pathname f-name))))
+             (or (null fdp)
+                 (and fdp               ; not simple file name.
+                      (not (eq (car (pathname-directory (pathname f-name)))
+                               :root))))))
+         (chaos-get-directory (file-path)
+           (declare (type (or simple-string pathname)))
+           (unless (pathnamep file-path)
+             (setq file-path (pathname file-path)))
+           (let ((dir-path (make-pathname :host (pathname-host file-path)
+                                          :device (pathname-device file-path)
+                                          :directory (pathname-directory file-path))))
+             dir-path)))
+    (if (null *chaos-input-source*)
+        (pathname f-name)
+      (if (chaos-relative-pathname?)
+          (let ((f-path nil))
+            (unwind-protect
+                (let ((host (pathname-host (pathname f-name)))
+                      (device (pathname-device (pathname f-name)))
+                      (fd (pathname-directory (pathname f-name)))
+                      (f (file-namestring (pathname f-name))))
+                  (chaos-pushd (chaos-get-directory *chaos-input-source*))
+                  #+GCL
+                   (setq f-path (truename (make-pathname :directory fd :name f)))
+                   #+:CLISP
+                    (setq f-path (make-pathname
+                                  :host host
+                                  :device device
+                                  :directory fd ;; (pathname fd)
+                                  :name f))
+                    #-(or GCL :CLISP)
+                     (progn
+                       (setq f-path (make-pathname
+                                     :host host
+                                     :device device
+                                     :directory fd
+                                     :name f))))
+              (chaos-popd))
+            f-path)
+        ;; absolute path or simple filename.
+        (pathname f-name)))))
 
 (defun chaos-probe-file (file load-path suffixes)
   (declare (type (or simple-string pathname) file)
-           (type (or simple-string list) load-path)
-           (type list suffixes))
-  (when (and (pathnamep file) (not (is-directory? file)))
-    (return-from chaos-probe-file (probe-file file)))
-  ;;
-  (setq file (expand-file-name file))
-  (when (atom suffixes)
-    (setq suffixes (list suffixes)))
-  (when (and load-path (atom load-path))
-    (setq load-path (list load-path)))
-  ;;
-  (cond ((is-relative-file-name? file)
-         (let ((file-path (chaos-get-relative-path*
-                           (concatenate 'string "./" file)))
-               (res nil))
-           ;;
-           (setq res (probe-file file-path))
-           (when (and res (is-directory? res))
-             (setq res nil))
-           (unless res
-             (dolist (fx (supply-suffixes file-path suffixes))
-               (when (and (probe-file fx) (not (is-directory? fx)))
-                 (setq res fx)
-                 (return)))
-             ;; search through load paths
+           (type list load-path suffixes))
+  (when (pathnamep file)
+    (if (is-directory? file)
+        (return-from chaos-probe-file nil)
+      (return-from chaos-probe-file (probe-file file))))
+  (let ((fp (expand-file-name file)))
+    (declare (type simple-string fp))
+    (cond ((is-relative-file-name? fp)
+           (let ((file-path (chaos-get-relative-path
+                             (concatenate 'string "./" fp)))
+                 (res nil))
+             (declare (type pathname file-path))
+             (setq res (probe-file file-path))
+             (when (and res (is-directory? res))
+               (setq res nil))
              (unless res
-               (dolist (lpath load-path)
-                 (let ((libdir (is-directory? lpath)))
-                   (declare (type (or null string pathname) libdir))
-                   (when libdir
-                     (unless (pathnamep libdir) (setq libdir (pathname libdir)))
-                     (let ((f (make-pathname
-                               :host (pathname-host libdir)
-                               :device (pathname-device libdir)
-                               :directory
-                               #+:CLISP libdir
-                               ;; #+:Allegro (namestring libdir)
-                               #-:CLISP (pathname-directory libdir)
-                               :name (namestring file))))
-                       (if (and (probe-file f) (not (is-directory? f)))
-                           (progn (setq res f) (return))
-                         (let ((x (supply-suffixes f suffixes)))
-                           (dolist (fx x)
-                             (when (and (probe-file fx) (not (is-directory? fx)))
-                               (setq res fx)
-                               (return)))))))))))
-           res))
-        (t (let ((file-path (chaos-get-relative-path* file)))
-             (if (and (probe-file file-path) (not (is-directory? file-path)))
-                 file-path
                (dolist (fx (supply-suffixes file-path suffixes))
                  (when (and (probe-file fx) (not (is-directory? fx)))
-                   (return-from chaos-probe-file fx))))))))
+                   (setq res fx)
+                   (return)))
+               ;; search through load paths
+               (unless res
+                 (dolist (lpath load-path)
+                   (declare (type pathname lpath))
+                   (let ((libdir (is-directory? lpath)))
+                     (declare (type (or null pathname) libdir))
+                     (when libdir
+                       (let ((f (make-pathname
+                                 :host (pathname-host libdir)
+                                 :device (pathname-device libdir)
+                                 :directory #+:CLISP libdir
+                                            #-:CLISP (pathname-directory libdir)
+                                 :name (namestring file))))
+                         (declare (type pathname f))
+                         (if (and (probe-file f) (not (is-directory? f)))
+                             (progn (setq res f) (return))
+                           (let ((x (supply-suffixes f suffixes)))
+                             (dolist (fx x)
+                               (when (and (probe-file fx) (not (is-directory? fx)))
+                                 (setq res fx)
+                                 (return)))))))))))
+             res))
+          (t (let ((file-path (chaos-get-relative-path file)))
+               (declare (type pathname file-path))
+               (if (and (probe-file file-path) (not (is-directory? file-path)))
+                   file-path
+                 (dolist (fx (supply-suffixes file-path suffixes))
+                   (when (and (probe-file fx) (not (is-directory? fx)))
+                     (return-from chaos-probe-file fx)))))))))
 
 (defun bare-chaos-pwd ()
   #+GCL (probe-file ".")
@@ -230,69 +259,6 @@
   (declare (values simple-string))
   (namestring (bare-chaos-pwd)))
 
-(defun chaos-relative-pathname? (f-name)
-  (let ((fdp (pathname-directory (pathname f-name))))
-    (or (null fdp)
-        (and fdp                        ; not simple file name.
-             (not (eq (car (pathname-directory (pathname f-name)))
-                      :root))))))
-
-(defun chaos-get-relative-path (f-name)
-  (setq f-name (expand-file-name f-name))
-  (chaos-get-relative-path* f-name))
-
-; #+:SBCL
-; (defun chaos-get-directory (file-path)
-;   (let* ((ns (namestring file-path))
-;        (dpos (position #\/ ns :from-end t))
-;        (dir nil))
-;     (unless dpos
-;       (with-output-chaos-error ('internal-error)
-;       (format t ":get-relative-path: could not find proper directory path, ~a" file-path)))
-;     (subseq ns 0 (1+ dpos))))
-
-#+(or :Allegro :SBCL)
-(defun chaos-get-directory (file-path)
-  (unless (pathnamep file-path)
-    (setq file-path (pathname file-path)))
-  (let ((dir-path (make-pathname :host (pathname-host file-path)
-                                 :device (pathname-device file-path)
-                                 :directory (pathname-directory file-path))))
-    ;;(namestring dir-path)
-    dir-path))
-
-(defun chaos-get-relative-path* (f-name)
-  (if (null *chaos-input-source*)
-      (pathname f-name)
-    (if (chaos-relative-pathname? f-name)
-        (let ((f-path nil))
-          (unwind-protect
-              (let ((host (pathname-host (pathname f-name)))
-                    (device (pathname-device (pathname f-name)))
-                    (fd (pathname-directory (pathname f-name)))
-                    (f (file-namestring (pathname f-name))))
-                ;; #-GCL (declare (ignore fd))
-                ;; (chaos-pushd (directory-namestring *chaos-input-source*))
-                (chaos-pushd (chaos-get-directory *chaos-input-source*))
-                #+GCL
-                (setq f-path (truename (make-pathname :directory fd :name f)))
-                #+:CLISP
-                (setq f-path (make-pathname
-                              :host host
-                              :device device
-                              :directory fd ;; (pathname fd)
-                              :name f))
-                #-(or GCL :CLISP)
-                (progn
-                  (setq f-path (make-pathname
-                                :host host
-                                :device device
-                                :directory fd
-                                :name f))))
-            (chaos-popd))
-          f-path)
-      ;; absolute path or simple filename.
-      (pathname f-name))))
 
 #+(or (and CCL (not :openmcl)) :microsoft)
 (defun chaos-ls (&optional (dir "**"))
@@ -386,11 +352,7 @@
         (pop *chaos-directory-stack*)
         (setq xd (car *chaos-directory-stack*))
         (chaos-cd xd))
-    ;; do nothig
-    #||
-    (with-output-chaos-warning ()
-      (format t "directory stack is empty!"))
-    ||#
+    ;; do nothing
     ))
 
 (defun chaos-cd (path &optional (always-return nil))
@@ -470,9 +432,8 @@
     (unless (or fname bin-fname)
       (if errorp
           (with-output-chaos-error ('no-such-file)
-            (format t "no such file: ~a" (namestring file)))
-        (return-from chaos-input-file nil)
-        ))
+            (format t "No such file: ~a" (namestring file)))
+        (return-from chaos-input-file nil)))
     (when (and bin-fname fname (>= (file-write-date bin-fname)
                                    (file-write-date fname)))
       (setq fname bin-fname))
@@ -491,11 +452,9 @@
             (with-output-chaos-warning ()
               (format t "input nesting is ~d~%" *chaos-input-level*)
               (print-next)
-              (princ "probable input loop (can increase *chaos-input-nesting-limit*)")
-              ))
+              (princ "probable input loop (can increase *chaos-input-nesting-limit*)")))
           (apply proc args)
-          fname
-          )))))
+          fname)))))
 ;;;
 (defun set-search-path (paths)
   (when (consp paths)
@@ -521,7 +480,7 @@
     (dolist (p path)
       (if (not (member p *chaos-libpath* :test #'equal))
           (with-output-chaos-warning ()
-            (format t "The path ~s does not in 'libpath'." p))
+            (format t "The path ~s is not in 'libpath'." p))
         (setq *chaos-libpath* (remove p *chaos-libpath* :test #'equal))))
     *chaos-libpath*))
 
